#include <stdio.h>
#include <assert.h>

#define MAX_MAT_SIZE 20

enum
{
  PIPE_HORIZONTAL = '',
  PIPE_VERTICAL = '',
  PIPE_CORNER_UL = '',
  PIPE_CORNER_UR = '',
  PIPE_CORNER_DL = '',
  PIPE_CORNER_DR = '',
  PIPE_CROSS = '',
  PIPE_NONE = ' ',
};

inline int isSymCorner(char c)
{
  return c == PIPE_CORNER_UL || c == PIPE_CORNER_UR || c == PIPE_CORNER_DL || c == PIPE_CORNER_DR;
}

int main (int argc, char** argv)
{
  char mat[MAX_MAT_SIZE][MAX_MAT_SIZE];
  memset(mat, PIPE_NONE, MAX_MAT_SIZE * MAX_MAT_SIZE);

  int occupiedRow[MAX_MAT_SIZE], occupiedCol[MAX_MAT_SIZE];
  memset(occupiedRow, 0, sizeof(occupiedRow));
  memset(occupiedCol, 0, sizeof(occupiedCol));
 
  int windingsToDo = atoi(argv[1]);
  srand(atoi(argv[2]));

  int startI = MAX_MAT_SIZE / 2;
  int startJ = MAX_MAT_SIZE / 2;

  int i = startI;
  int j = startJ;
  occupiedRow[i] = 1;
  occupiedCol[j] = 1;

  int quarterWindingsDone;
  for (quarterWindingsDone = 0; quarterWindingsDone < 4 * windingsToDo; ++quarterWindingsDone)
  {
    int *activeResourceOccupation = (quarterWindingsDone % 2 == 0) ? occupiedCol : occupiedRow;
    int coordinateRangeL, coordinateRangeU;
    if (quarterWindingsDone % 4 == 0) { coordinateRangeL = startJ + 1; coordinateRangeU = MAX_MAT_SIZE; }
    if (quarterWindingsDone % 4 == 2) { coordinateRangeL = 0; coordinateRangeU = startJ; }
    if (quarterWindingsDone % 4 == 1) { coordinateRangeL = startI + 1; coordinateRangeU = MAX_MAT_SIZE; }
    if (quarterWindingsDone % 4 == 3) { coordinateRangeL = 0; coordinateRangeU = startI; }

    int usePos;
    if (quarterWindingsDone < 4 * windingsToDo - 2)
    {
      int freePoses = 0;
      int freePosIdxes[MAX_MAT_SIZE];
      int c;
      for (c = coordinateRangeL; c < coordinateRangeU; ++c) 
        if (!activeResourceOccupation[c])
        {
          freePosIdxes[freePoses] = c;
          freePoses++;
        }

      usePos = freePosIdxes[rand() % freePoses];
      activeResourceOccupation[usePos] = 1;
    }
    else
    {
      if (quarterWindingsDone == 4 * windingsToDo - 2) usePos = startJ;
      if (quarterWindingsDone == 4 * windingsToDo - 1) usePos = startI;
    }

    int* activeCoordinate = (quarterWindingsDone % 2 == 0) ? &j : &i;

    while (*activeCoordinate != usePos)
    {
      if (usePos < *activeCoordinate) (*activeCoordinate)--; else (*activeCoordinate)++;
      assert(!isSymCorner(mat[i][j]));
      mat[i][j] = (mat[i][j] == PIPE_HORIZONTAL || mat[i][j] == PIPE_VERTICAL) ? PIPE_CROSS : ((quarterWindingsDone % 2 == 0) ? PIPE_HORIZONTAL : PIPE_VERTICAL);
    }
    if (quarterWindingsDone % 4 == 0) { mat[i][j] = PIPE_CORNER_DL; }
    if (quarterWindingsDone % 4 == 1) { mat[i][j] = PIPE_CORNER_UL; }
    if (quarterWindingsDone % 4 == 2) { mat[i][j] = PIPE_CORNER_UR; }
    if (quarterWindingsDone % 4 == 3) { mat[i][j] = PIPE_CORNER_DR; }
       
  } 

  printf ("%d %d\n", MAX_MAT_SIZE, MAX_MAT_SIZE);
  for (i = 0; i < MAX_MAT_SIZE; ++i)
  {
    for (j = 0; j < MAX_MAT_SIZE; ++j) printf ("%c", mat[i][j]);      
    printf("\n");
  }
  
  return 0;
}
